#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ref_targ.tcl,v 2.43 2004/06/17 23:04:34 vltsccm Exp $"
#
# who        when        what
# --------   --------    ----------------------------------------------
# W.Hummel   11.Nov.98   created 
# W.Hummel   23.Feb.99   purge and purge_single 
# aaguayo    05/05/01    modified verify method to be used also when loading 
#                        a setup file
# aaguayo    18/05/02    removed changes bellow (verify method) as for version 2.29
# mkiekebu  2002/10/05   included changes removed previous version
# mkiekebu 2004-06-14    Fixed problem with dialog windows (RH9.0)


#************************************************************************
#   NAME             
#   REF_TARG - Tcl class           
# 
#   NAMESPACE
#       fors
#
#   SYNOPSIS         
#       REF_TARG <instance_name> canvas image ?-eqx <equinox>?
#       
#   PARENT CLASS
#       none 
#
#   DESCRIPTION      
#       Handling of the reference targets for alignment 
#
#   OPTIONS
#
#   PUBLIC METHODS
#       plot  { radius } 
#             plot all reference targets on the canvas
#             2 cyan circle, two black circles, a cyan and a 
#             black number label 
#       get   { i indent }
#       put   { a d }
#       reset {} 
#             purge the complete list of reference stars
#       purge {}
#             purge only a fraction of reference stars
#       show  {}
#             shows ref-targ numbers and R,DEC in a list_box
#       nort  {}
#             returns the number of reference targets [0..10]
#       test  {}
#       add   {}
#      verify {}
#             checks if reference target positions are inside the
#             focal field and are bout of the forbidden regions
#
#   PUBLIC DATA MEMBERS
#       ins_path    $env(INS_ROOT)
#       int_path    $env(FIMSROOT)
#       iu_dir      $env(INS_USER)
#       eqx         2000.0
#
#   PROTECTED METHODS
#
#   PROTECTED DATA MEMBERS
#
#   PRIVATE METHODS
#       purge_single { j }   
#       put2 { lll }
#
#   PRIVATE DATA MEMBERS
#       num
#       ref_ra,ref_de,rel
#       m_canvas,m_image  
#
#   COMMANDS
#       REF_TARG C_ref_targ  { m_image m_canvas equinox} 
#
#   RETURN VALUES
#
#   CAUTIONS
#       Use FIMS V1.1.5 only with skycat V2.1.3
#
#   EXAMPLES
#
#   SEE ALSO
#
#------------------------------------------------------------------------
#
#23456789 123456789 123456789 123456789 123456789 123456789 123456789 1234567890
#


itk::usual FORS {}

# create a class for the application

namespace eval fors {

class REF_TARG {

#------------------------
constructor { m_c m_i args } {
#------------------------
     # puts stdout "$this: constructor"
     set m_image  $m_i
     set m_canvas $m_c
     set num 0

     for {set i 1} { $i <=10} { incr i 1} {
         set ref($i,a) 0.0
         set ref($i,d) 0.0
         }
     eval configure $args
     }

     #---------------
     destructor {
     #---------------
      # puts stdout "$this: destructor"
      }

     #============================
     public method get { i ident } {
     #============================
     # ident must be 'a' or 'd' 
     # i must be [1..10]
     if {$i>$num} return
     if {$i<1} return 
     return $ref($i,$ident)
     }


     #============================
     public method add {} {
     #============================
     set cur $this

     if {$num<10} {

     info_dialog "Add up to [expr 10-$num] reference targets\n\
                  --isolated point source\n\
                  --bright, not saturated\n\
                  --avoid cyan area\n\
                  --\[Enter\] empty string to stop" .skycat1


        
        set diff 1
        while {$num<10 && $diff} {
        set diff 0 
        # puts stdout ">$num< >[$cur get $num a]< \
        #    >[$cur get [expr $num-1] a]< >$diff<"

        EnterObject .eo \
           -title "Reference Object #[expr $num+1]" \
           -image .skycat1.image \
           -labels {ra dec} \
           -command  [code $cur put2]

        tkwait window .eo

        }
        }
    }

     #============================
     private method put2 { lll } {
     #============================
     $this put [lindex $lll 0] [lindex $lll 1]
     $this plot 20
     set diff 1
     } 

     #============================
     public method put { a d } {
     #============================
     # puts stdout "$this: put $a $d"
        if {$num==10} {
           puts stdout "$this put: Error array full"
           } else {
           set ref([expr $num+1],a) $a
           set ref([expr $num+1],d) $d
           incr num 1
           }
       }

     #============================
     public method purge {} {
     #============================
     # j might be a list like 5,1,8
     # 2-4 syntax not provided
     set text "Purge Reference Targets \n\n\ "

     if {$num==0} {
        append text "\nNothing to Purge" 
        } else {
     for {set i 1} {$i <=$num} { incr i 1} {
         append text [format \
         "#%2d   %s   %s\n\ " $i $ref($i,a) $ref($i,d) ]
         }
     append text "\nPress \[Ok\] or Enter e.g. 1,$num\n"
       }


      if {$num==0} {
         info_dialog $text .skycat1
         } else {

     set wid .skycat1.d         
     catch {destroy $wid}
     set w [InputDialog $wid \
              -text $text \
              -bitmap warning \
              -modal 1 \
              -buttons {OK Cancel}]

     set loc_list [split [$w activate] , ]

     # we first have to purge the highest order number
     set new_list [lsort -integer -decreasing $loc_list]

     for {set i 0} {$i < [llength $new_list]} { incr i 1 } {
        $this purge_single [lindex $new_list $i]
        }
      }
  } 
 

#-------------------------------------
private method ldelete_all { w_list value} {
#-------------------------------------
        # modified ldelete
        # delete list elements by value
        set ix [lsearch -exact $w_list $value]
        set new $w_list

        while {$ix>=0} {
               set new [lreplace $w_list $ix $ix]
               set ix [lsearch -exact $new $value]
               set w_list $new
               }
        return $new
	   }


     #============================ 
     public method verify { { par_load 0 } } {
     #============================
     # two tests are performed:
     # test1 : if several reference targets are identical
     # test2 :
     # we test if the position of the reference targetes 
     # are usefull. The cyan forbidden areas must be 
     # drawn on the canvas when calling this method
     # In order to be valid the RT position must  not
     # match the blue border (the CCD rim for HR/SR collimator)
     # must not match the forbidden cyan area 

	 set text "Verify Positions of \n$num Reference Targets:\n\n"
	 set t ""

	 if {$num==0} {
	     info_dialog "No Reference Targets\nVerification Result:\n\n   Failed" .skycat1
	     return 1
         }

	 for {set i 1} {$i <= $num} { incr i 1} {
	     set inside_mask_flag 0; # default outside the focal field 
	     set not_in_zone_flag 1; # default outside forbidden zone
	     $m_image convert coords $ref($i,a) $ref($i,d) "wcs $eqx" \
		     x y canvas

	     set a [$m_canvas find overlapping \
		     [expr $x-1] [expr $y-1] \
		     [expr $x+1] [expr $y+1] ]

	     foreach c $a {
		 set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
		 # puts stdout ">$local_tags<"

		 if {[string compare [lindex $local_tags 0] A]==0} {
		     # reftarg inside the focal field
		     set inside_mask_flag 1
		 }

		 if {[string compare [lindex $local_tags 2] rtz]==0} {
		     # reftarg inside the forbidden zone 
		     set not_in_zone_flag  0
		 }
	     } 

	     # this was test2 on the forbidden zone 
	     # it follows test1 on the dublicate appearance 
	     # of reference targets 
	     set multi ""
	     if {$i<$num} {
		 for {set j [expr $i +1] } {$j <= $num} { incr j 1} {
		     $m_image convert coords $ref($i,a) $ref($i,d) "wcs $eqx" \
			     lra1 lde1 "deg $eqx"
		     $m_image convert coords $ref($j,a) $ref($j,d) "wcs $eqx" \
			     lra2 lde2 "deg $eqx"
		     set t1 [expr $lra1 -$lra2]; set t2 [expr $lde1 -$lde2]
		     # distance in arcsec 
		     set t3 [expr sqrt($t1*$t1 + $t2*$t2)*3600.0 ] 
		     if {$t3<2.0} { append multi " = #$j" }
		 }
             }
	     # test1 finished 

	     if {$inside_mask_flag && $not_in_zone_flag} {
		 if {$multi != ""} { 
		     append text " #$i $multi\n"
		     if {$t == ""} { append t $i } else { append t ",$i" }
		 } else {
		     append text " #$i Ok\n" 
		 }
	     } else {
		 append text " #$i Not Accepted\n"
		 if {$t == ""} { append t $i } else { append t ",$i" }
	     }
         }


	 if {$t==""} {
	     append text "\nPress \[Ok\]\n"
	     info_dialog "Reference Targets\nVerification Result:\n\n     Ok" .skycat1
	 } else { 
	     append text "\nEnter $t and \nPress \[Ok\] to purge\n"

	     # the rest is the same as in the purge method 	

	     if { $par_load } {
		 # verify when loading
		 info_dialog "There are target references: $t \n In Forbidden Zone!! \n Removing ..." .skycat1
		 set loc_list [split $t , ]
		 set new_list [lsort -integer -decreasing $loc_list]
		 for {set i 0} {$i < [llength $new_list]} { incr i 1 } {
		     $this purge_single [lindex $new_list $i]
		 }
		 $this plot 1.0
		 return 0
	     } else {   
                 puts "FLAG ..."
		 # normal verify - user is aked to purge wrong ref targets	     
                 set wid .skycat1.d          
                 catch {destroy $wid}
		 set w [InputDialog $wid \
			 -text $text \
			 -bitmap warning \
			 -modal 1 \
			 -buttons {OK Cancel}]

		 set result [$w activate]
		 set loc_list [split $result , ]

		 # we first have to purge the highest order number
		 set new_list [lsort -integer -decreasing $loc_list]

		 for {set i 0} {$i < [llength $new_list]} { incr i 1 } {
		     $this purge_single [lindex $new_list $i]
		 }
		 $this plot 1.0

		 if {[string compare $result $t] || $num==0} { 
		     # if old==new (nothing edited) or no ref targ specified 
		     info_dialog "Reference Targets\nVerification Result:\n\n   Failed" .skycat1
		     return 1
		 } else {
		     info_dialog "Reference Targets\nVerification Result:\n\n     Ok" .skycat1
		     return 0
		 } 
	     }
	 }
     }
     

     #============================
     private method purge_single { j } {
     #============================
     # puts stdout "$this: purge_single $j "
        if {$j<1 || $j>$num} {
           # puts stdout "$this purge_single: $j not in 1..$num"
           } else {
           for {set i $j} {$i < $num} {incr i 1} {
                set ref($i,a) $ref([expr $i+1],a)
                set ref($i,d) $ref([expr $i+1],d)
                }
           if {$num>0} { incr num -1 }
           }
       # puts stdout "new number of ref_targ : $num"
       }

       
     #========================
     public method plot { radius } {
     #========================
     # puts stdout "$this: plot"
     catch {$m_canvas delete REFTARG } 
     set radius2 [expr $radius * 0.95]
         set sadius  [expr $radius / 5.0 ]
         set sadius2 [expr $radius2 / 5.0 ]
     for {set i 1} {$i <= $num} { incr i 1} {
         $m_image convert coords $ref($i,a) $ref($i,d) "wcs $eqx" \
                  x y canvas

         $m_canvas create oval [expr $x-$radius] [expr $y-$radius] \
                          [expr $x+$radius] [expr $y+$radius] \
                          -outline cyan -tag " REFTARG ref$i "
         $m_canvas create oval [expr $x-$radius2] [expr $y-$radius2] \
                          [expr $x+$radius2] [expr $y+$radius2] \
                          -outline black -tag " REFTARG ref$i "

         $m_canvas create oval [expr $x-$sadius] [expr $y-$sadius] \
                          [expr $x+$sadius] [expr $y+$sadius] \
                          -outline cyan -tag " REFTARG ref$i "
         $m_canvas create oval [expr $x-$sadius2] [expr $y-$sadius2] \
                          [expr $x+$sadius2] [expr $y+$sadius2] \
                          -outline black -tag " REFTARG ref$i "

         $m_canvas create text $x $y -fill cyan -tag " REFTARG ref$i " \
                          -anchor w -text [ format "  %2d" $i ]
         $m_canvas create text $x $y -fill black -tag " REFTARG ref$i " \
                          -anchor e -text [ format "%2d  " $i ]
         }

     }

     #========================
     public method reset {} {
     #========================
     # puts stdout "$this: reset"
     for {set i 1} { $i <=10} { incr i 1} {
         set ref($i,a) 0.0
         set ref($i,d) 0.0
         }
     set num 0
     $this plot 1.0; # delete them all from the canvas
     }

     #========================
     public method show {} {
     #========================
     # puts stdout "$this: show"
     set text "Reference Targets\n\n\ "
     if {$num>0} {
     for {set i 1} {$i <=$num} { incr i 1} {
         append text [format "#%2d   %s   %s\n\ " $i $ref($i,a) $ref($i,d) ]
         }
     } else {
     append text "no targets selected\n\ "
     }
     info_dialog $text .skycat1
     }

     #========================
     public method nort {} {
     #========================
     # puts stdout "$this: nort $num"
     return $num
     }

     #========================
     public method test {} {
     #========================
     puts stdout [$this nort]
     $this show  
     for {set i 1 } {$i<=11} {incr i 1} {
         $this put 00:00:30.3 -72:34:34.5
         $this show
         puts stdout [$this nort]
         puts stdout "[$this get $i a] [$this get $i d]"
         }
    
     }

     #========================
     proc init_ref_targ {} {
     #========================
          REF_TARG ref_targ
          ref_targ test
          ref_targ show
          ref_targ destructor
          }

      public variable eqx 2000.0
      public variable mos10ali 0  

      private variable ref_ra 0.0
      private variable ref_de 0.0
      private variable ref 
      private variable num 0;  # NumberOfReferenceTargets
      private variable diff

      private variable m_canvas
      private variable m_image
    }
     # end of the class REF_TARG

}
# end of namespace

# fors::REF_TARG::init_ref_targ
#
# ___oOo___

